{% extends "base.html" %}

{% block extrahead %}
  {{ super() }}
  <meta name="description" content="{{ config.site_description | e }}" />
  <meta property="og:site_name" content="{{ config.site_name | e }}" />
  <meta property="og:title" content="{{ (page.title ~ ' - ' ~ config.site_name) | e }}" />
  <meta property="og:description" content="{{ config.site_description | e }}" />
  <meta property="og:url" content="{{ page.canonical_url | e }}" />
  <script>
    window.__MERLYA_RAG_API_URL = {{ config.extra.get('rag_api_url') | default('', true) | tojson }};
  </script>
{% endblock %}

